/*  2014-2015 modified by Erinome LLC
 *  gtsOverlay.js      (c) Till Wiebke
 *  Code for for the browser-overlay (gtsOverlay.xul) of "Go to Selection"
 *  Last changed: 2015-03-10
 *  http://erinome.net
 */

// bundle holding the localized strings
var localizedStrings;

function gts_init() {
	// default search engine
	if (!gts_prefs.prefHasUserValue("defaultSearchEngine"))
	{
		gts_prefs.setCharPref("defaultSearchEngine",gts_se.defaultEngine.name);
	}
	// Eventlistener for the contextmenu
	var menu = document.getElementById("contentAreaContextMenu");
	menu.addEventListener("popupshowing", gts_showMenu, false);
	// get localized strings from properties-file
	localizedStrings = document.getElementById("strings");
	document.getElementById("gotowebaddress-tab").label = localizedStrings.getString("GoToSelection");
	// modify search context menu
	var searchMenuItem, searchMenuPopup, searchSubMenuItem;
	searchMenuItem = document.createElement("menuseparator");
	searchMenuItem.id = "gtwa-search-sep";
	menu.insertBefore(searchMenuItem,menu.firstChild);

	searchMenuItem = document.createElement("menu");
	searchMenuItem.id = "gtwa-searchin-tab";
	searchMenuItem.setAttribute("label",localizedStrings.getString("SearchIn"));

	searchMenuPopup = document.createElement("menupopup");

	gts_se.getVisibleEngines({}).forEach(function(engine) {
		searchSubMenuItem = document.createElement("menuitem");
		searchSubMenuItem.id = "gtwa-searchin-"+engine.name;
		searchSubMenuItem.setAttribute("label",engine.name);
		if (engine.iconURI !== null)
		{
			searchSubMenuItem.setAttribute("class","menuitem-iconic");
			searchSubMenuItem.setAttribute("image",engine.iconURI.spec);
		}
		searchSubMenuItem.addEventListener("command", function(e){gts_searchThisIn(e,engine.name);}, false);
		searchMenuPopup.appendChild(searchSubMenuItem);
	});

	searchMenuItem.appendChild(searchMenuPopup);
	menu.insertBefore(searchMenuItem,menu.firstChild);

	searchMenuItem = document.createElement("menuitem");
	searchMenuItem.id = "gtwa-search-tab";
	searchMenuItem.setAttribute("label",localizedStrings.getString("SearchThis"));
	if (gts_se.currentEngine.iconURI !== null)
	{
		searchMenuItem.setAttribute("class","menuitem-iconic");
		searchMenuItem.setAttribute("image",gts_se.currentEngine.iconURI.spec);
	}
	searchMenuItem.addEventListener("command", gts_searchThis, false);
	menu.insertBefore(searchMenuItem,menu.firstChild);
}

// Function to display and set the labels of the menuitems
function gts_showMenu()
{
	if(gContextMenu.isTextSelected)
	{
		document.getElementById("gtwa-search-sep").hidden = false;
		document.getElementById("gtwa-search-tab").hidden = false;
		document.getElementById("gtwa-searchin-tab").hidden = false;

		document.getElementById("gotowebaddress-sep").hidden = false;
		document.getElementById("gotowebaddress-tab").hidden = false;
	}
	else
	{
		document.getElementById("gtwa-search-sep").hidden = true;
		document.getElementById("gtwa-search-tab").hidden = true;
		document.getElementById("gtwa-searchin-tab").hidden = true;

		document.getElementById("gotowebaddress-sep").hidden = true;
		document.getElementById("gotowebaddress-tab").hidden = true;
	}
	// hide default
	document.getElementById("context-searchselect").hidden = true;
}

// standard content.getSelection() requires us to specify where exactly user has selected a text;
// if that happens in main window then everything works as expected, but when iframes are being used it breaks,
// thus we need a custom getSelection() method
function gts_getSelection()
{
	var focusedWindow = document.commandDispatcher.focusedWindow;
	if (focusedWindow == window) focusedWindow = content;
	var str  = focusedWindow.getSelection().toString();
	str = str.replace(/^\s*|\s*$/g, '');
	str = str.replace(/\s+/g," ");
	return str;
}

function gts_searchThisIn(evnt,sename)
{
	gBrowser.selectedTab.gts_cse = sename;
	var this_se = gts_se.getEngineByName(sename);
	if (this_se.iconURI !== null)
	{
		document.getElementById("gtwa-search-tab").setAttribute("class","menuitem-iconic");
		document.getElementById("gtwa-search-tab").setAttribute("image",this_se.iconURI.spec);
	}
	else
	{
		document.getElementById("gtwa-search-tab").setAttribute("class","");
		document.getElementById("gtwa-search-tab").setAttribute("image","");
	}

	var selection = gts_getSelection(content);
	var uri = this_se.getSubmission(selection).uri;

	gts_redirect(uri,evnt);
}

function gts_searchThis(evnt)
{
	var selection = gts_getSelection(content);
	var uri;
	if (!gBrowser.selectedTab.gts_cse)
		uri = gts_se.currentEngine.getSubmission(selection).uri;
	else
		uri = gts_se.getEngineByName(gBrowser.selectedTab.gts_cse).getSubmission(selection).uri;

	gts_redirect(uri,evnt);
}
// Function to open URLs
function gts_gotoURL(evnt) {
	var uri = null;
	var selection = gts_getSelection(content);

	if (!gts_isUrl(selection))
	{
		if (!gts_isUrl('http://'+selection))
		{
			uri = gts_se.currentEngine.getSubmission(selection).uri;
			selection = false;
		}
		else
		{
			selection = 'http://'+selection;
		}
	}

	if (selection !== false)
		uri = gts_stringToUrl(selection);

	if (uri == null)
	{
		window.alert(localizedStrings.getString("ConversionError") + selection);
		return;
	}

	gts_redirect(uri,evnt);
}

// Helper for making nsURI from string
function gts_stringToUrl(spec)
{
  var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
  var newUri = null;
  try
  {
		newUri = ios.newURI(spec, null, null);
  }
  catch(err)
  {
    // Do nothing
  }
  return newUri
}

// actual action
function gts_redirect(uri,evnt)
{
	var rtarget;
	if (evnt.ctrlKey)
		rtarget = gts_prefs.getIntPref("ctrlclick");
	else if (evnt.shiftKey)
		rtarget = gts_prefs.getIntPref("shiftclick");
	else
		rtarget = gts_prefs.getIntPref("singleclick");

	switch (rtarget)
	{
		case 0:
			openUILinkIn(uri.spec,"current");
			break;
		case 1:
			openUILinkIn(uri.spec,"tab");
			break;
		case 2:
			openUILinkIn(uri.spec,"tabshifted");
			break;
	}
}

// regexp to check if selection is looking like url or not
function gts_isUrl(s) {
	var regexp = /^(https?|ftp|about|chrome):\/\/[^\s\/$.?#].[^\s]*$/
	return regexp.test(s);
}

function gts_searchBarExt(win)
{
	var self      = this;
	var window    = win;
	var searchbar = win.document.getElementById("searchbar");

	this.gtsHandleSearch = null;
	this.gtsDoSearch = null;
	this.gtsToolboxCustomizeDone = null;
	this.eventModifier = false;
	this.eventShift = false;
	
	this.gtsToolboxCustomizeDone = window.BrowserToolboxCustomizeDone;
	window.BrowserToolboxCustomizeDone = function(aToolboxChanged){
		self.gtsToolboxCustomizeDone.apply(this,arguments);
		self.init();
	};
	
	var toolbox = window.document.getElementById("navigator-toolbox");
	toolbox.customizeDone = window.BrowserToolboxCustomizeDone;
	
	this.init = function()
	{
		if(!searchbar)
			return;
	
/*	    if( searchbar._popup && !window.document.getElementById("searchloadoptions-menuitem") )
	    {
	        let element = window.document.createElement("menuitem");
	        element.setAttribute("class","open-engine-manager");
	        element.setAttribute("id","searchloadoptions-menuitem");
	        element.setAttribute("label",stringBundle.getString("searchoptions.label"));
	        element.addEventListener('command', this.configure, true );
	        searchbar._popup.appendChild(element);
	    }*/
	
		if (!this.gtsHandleSearch)
		{
			this.gtsHandleSearch = searchbar.handleSearchCommand;
			if (this.gtsHandleSearch) {
				searchbar.handleSearchCommand = function(aEvent,aEngine){
					self.eventModifier = aEvent && (aEvent.altKey || aEvent.ctrlKey || aEvent.shiftKey || aEvent.button==1);
					self.eventShift    = aEvent && aEvent.shiftKey;
					self.gtsHandleSearch.apply(this,arguments);
					self.eventModifier = false;
					self.eventShift    = false;
				};
			}
		}

		if (!this.gtsDoSearch)
		{
			this.gtsDoSearch = searchbar.doSearch;
			if (this.gtsDoSearch) {
				searchbar.doSearch = function(aData,aWhere,aEngine)
				{
					var destTabCode;
					if (self.eventShift)
						destTabCode = gts_prefs.getIntPref("shiftclick");
					else if (self.eventModifier)
						destTabCode = gts_prefs.getIntPref("ctrlclick");
					else
						destTabCode = gts_prefs.getIntPref("singleclick");
	
					switch (destTabCode)
					{
						case 0: aWhere = "current";			break;
						case 1: aWhere = "tab";					break;
						case 2: aWhere = "tabshifted";	break;
					}
	        
	        if (aEngine)
	        {
	        	gts_se.currentEngine = aEngine;
	        	gBrowser.selectedTab.gts_sbse = aEngine.name;
						// update icon for context menu search if there wasn't custom se chosen on target tab
						if (!gBrowser.selectedTab.gts_cse) {
							if (aEngine.iconURI !== null)
							{
								document.getElementById("gtwa-search-tab").setAttribute("class","menuitem-iconic");
								document.getElementById("gtwa-search-tab").setAttribute("image",aEngine.iconURI.spec);
							}
							else
							{
								document.getElementById("gtwa-search-tab").setAttribute("class","");
								document.getElementById("gtwa-search-tab").setAttribute("image","");
							}
						}
	        }
	        
					self.gtsDoSearch.apply(this, arguments);
				};
			}
		}
		
		searchbar.addEventListener("input", gts_sbInputReceiver, false);

	};//init
}

// update searchbar value for current tab
function gts_sbInputReceiver()
{
	var searchbar = document.getElementById("searchbar");
	gBrowser.selectedTab.gts_sbct = searchbar.value;
}

// process tab switching
function gts_onTabChange()
{
	var this_se;
	var searchbar = document.getElementById("searchbar");

	// in case there's no searchbar
	if (!searchbar)
	{
		return;
	}

	// switch search engine in searchbar
	if (gBrowser.selectedTab.gts_sbse)
	{
		this_se = gts_se.getEngineByName(gBrowser.selectedTab.gts_sbse);
		if (this_se == null)
		{
			gBrowser.selectedTab.gts_sbse = gts_prefs.getCharPref("defaultSearchEngine");
			this_se = gts_se.getEngineByName(gBrowser.selectedTab.gts_sbse);
			if (this_se == null)
			{
				gBrowser.selectedTab.gts_sbse = gts_se.defaultEngine.name;
				this_se = gts_se.defaultEngine;
			}
		}
		gts_se.currentEngine = this_se;
	}
	else
	{
		this_se = gts_se.getEngineByName(gts_prefs.getCharPref("defaultSearchEngine"));
		if (this_se == null)
		{
			gts_prefs.setCharPref("defaultSearchEngine",gts_se.defaultEngine.name);
			this_se = gts_se.defaultEngine;
		}
		gts_se.currentEngine = this_se;
	}
	
	// switch search term in searchbar
	if (gBrowser.selectedTab.gts_sbct)
	{
		searchbar.value = gBrowser.selectedTab.gts_sbct;
	}
	else
	{
		searchbar.value = '';
	}

	// switch search engine for context menu (it may differ from tab search engine!)
	if (gBrowser.selectedTab.gts_cse)
		this_se = gts_se.getEngineByName(gBrowser.selectedTab.gts_cse);
	else
		this_se = gts_se.currentEngine;

	// switch icon in context menu search
	if (this_se.iconURI !== null)
	{
		document.getElementById("gtwa-search-tab").setAttribute("class","menuitem-iconic");
		document.getElementById("gtwa-search-tab").setAttribute("image",this_se.iconURI.spec);
	}
	else
	{
		document.getElementById("gtwa-search-tab").setAttribute("class","");
		document.getElementById("gtwa-search-tab").setAttribute("image","");
	}

	// update search bar display
	searchbar._textbox.popup.updateHeader();
	searchbar.updateDisplay();
}

// Get the preferences-branch of gts
const gts_prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.gotowebaddress.");
const gts_se = Components.classes["@mozilla.org/browser/search-service;1"].getService(Components.interfaces.nsIBrowserSearchService);

// Initialize the overlay when a document is loaded
window.addEventListener("load", gts_init, false);

// Initialize search bar extension part
window.gts_searchBarExt = new gts_searchBarExt(window);
window.gts_searchBarExt.init();

// Map some actions to tabchange event
window.addEventListener("load", function(e) {
	gBrowser.tabContainer.addEventListener("TabSelect", gts_onTabChange, false);
}, false);

window.addEventListener("unload", function(e) {
	gBrowser.tabContainer.removeEventListener("TabSelect", gts_onTabChange, false);
}, false);